<?php

namespace app\commands;

use yii\console\Controller;
use yii;
use yii\helpers\ArrayHelper;
use app\components\purchases\ClientSiesa;
use app\models\Fulfillmentprovision;

class FullfillmentProviderController extends Controller {

    /**
     *
     */
    public function actionIndex() {
        Yii::$app->db->createCommand('TRUNCATE data_fulfillment_provider')->execute();
        $this->nsFullfillmentProvision();
    }

    /**
     * Metodo para extraer e insertar localmente la informacion de siesa 
     * para el calculo del nivel de servicio de aprovisionamiento
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function nsFullfillmentProvision() {
        $dateEnd = date('d-m-Y');
        $dateIn = date('d-m-Y', strtotime('-1 month'));
        $result = $this->_getData($dateIn, $dateEnd);

        if (count($result) > 0) {
            $arrayBlocks = array_chunk($result, 500);
            foreach ($arrayBlocks as $block) {
                $this->_insertData($block);
            }
        }
    }

    /**
     * Metodo para la insercion en bloque de las referencias
     * segun el proveedor
     * 
     * @param array $data
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _insertData($data) {
        try {
            if (empty($data) == false) {
                Yii::$app->db->createCommand()->batchInsert('data_fulfillment_provider',
                        [
                            'year_purchase',
                            'year_entry',
                            'month_purchase',
                            'month_entry',
                            'document_type_purchase',
                            'document_type_entry',
                            'number_purchase',
                            'date_purchase',
                            'number_entry',
                            'date_entry',
                            'status_purchase',
                            'nit',
                            'business_name',
                            'branch_code',
                            'branch_name',
                            'reference',
                            'description',
                            'total_references',
                            'total_ref_inc',
                            'quantity_purchase',
                            'unit_value_purchase',
                            'quantity_entry',
                            'unit_value_entry',
                            'total_unit_value_purchase',
                            'subtotal_purchase',
                            'subtotal_entry',
                            'total_purchase',
                            'total_entry',
                            'operation_center',
                           ],
                        $data)->execute();
            }
        } catch (\Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r($exc->getMessage(), true) . '}');
        }
    }

    /**
     * Metodo para obtener la informacion
     * 
     * @param string $dateIn  Fecha Inicial fotmato(dd-mm-YYYY)
     * @param string $dateEnd Fecha Final fotmato(dd-mm-YYYY)
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getData($dateIn, $dateEnd) {
        $arrayResult = array();
        try {
            $sql = "
                SELECT
                    YEAR(f420_fecha) as year_purchase,
                    YEAR(f350_fecha) as year_entry,
                    DATENAME(M, DATEADD(M, MONTH(f420_fecha), -1)) as month_purchase,
                    DATENAME(M, DATEADD(M, MONTH(f350_fecha), -1)) as month_entry,
                    T420.f420_id_tipo_docto AS document_type_purchase,
                    t350_t1.f350_id_tipo_docto AS document_type_entry,
                    T420.f420_consec_docto AS number_purchase,
                    T420.f420_fecha AS date_purchase,
                    t350_t1.f350_consec_docto AS number_entry,
                    t350_t1.f350_fecha AS date_entry,
                    T420.f420_ind_estado AS status_purchase,
                    PROVEEDOR.f200_id AS nit,
                    LTRIM(RTRIM(PROVEEDOR.f200_razon_social)) AS business_name,
                    LTRIM(RTRIM(T420.f420_id_sucursal_prov)) AS branch_code,
                    f201_descripcion_sucursal AS branch_name,
                    LTRIM(RTRIM(T120.f120_referencia)) AS reference,
                    LTRIM(RTRIM(T120.f120_descripcion)) AS description,
                    COUNT(f120_referencia) OVER() AS total_references,
                    SUM(CASE WHEN f421_cant_pedida <> f421_cant_entrada THEN 1 ELSE 0 END) AS total_ref_inc,
                    SUM(T421.f421_cant_pedida) AS quantity_purchase,
                    SUM(t421.f421_precio_unitario) AS unit_value_purchase,
                    SUM(T421.f421_cant_entrada) AS quantity_entry,
                    SUM(t421.f421_precio_unitario) AS unit_value_entry,
                    SUM(f421_vlr_bruto) OVER() AS total_unit_value_purchase,
                    SUM(f421_cant_pedida * f421_precio_unitario) AS subtotal_purchase,
                    SUM(f421_precio_unitario * f421_cant_entrada) AS subtotal_entry,
                    SUM(f421_cant_pedida * f421_precio_unitario) OVER() AS total_purchase,
                    SUM(f421_precio_unitario * f421_cant_entrada) OVER() AS total_entry,
                    T420.f420_id_co as operation_center
                FROM
                    (t157_mc_instalaciones T157 RIGHT OUTER JOIN
                    (t150_mc_bodegas T150 RIGHT OUTER JOIN
                     (t200_mm_terceros PROVEEDOR RIGHT OUTER JOIN
                      (t120_mc_items T120 RIGHT OUTER JOIN
                           (t121_mc_items_extensiones T121 RIGHT OUTER JOIN
                            (t420_cm_oc_docto T420 INNER JOIN t421_cm_oc_movto T421 
                             ON T420.f420_id_cia = T421.f421_id_cia
                             AND T420.f420_rowid = T421.f421_rowid_oc_docto)
                            ON T121.f121_rowid = T421.f421_rowid_item_ext)
                           ON T121.f121_rowid_item = T120.f120_rowid)
                      ON PROVEEDOR.f200_rowid = T420.f420_rowid_tercero_prov)
                     ON T150.f150_rowid = T421.f421_rowid_bodega)
                    ON T157.f157_id = T150.f150_id_instalacion
                    AND T157.f157_id_cia = T150.f150_id_cia)

                left JOIN T201_MM_CLIENTES C201
                    ON PROVEEDOR.F200_ROWID = C201.F201_ROWID_TERCERO
                    AND T420.f420_id_sucursal_prov = C201.F201_ID_SUCURSAL  

                Left Join t451_cm_docto_compras T451
                    ON t420.f420_rowid = t451.f451_rowid_oc_docto

                Left Join t350_co_docto_contable t350_t1 
                    ON t451.f451_rowid_docto = t350_t1.f350_rowid 

                INNER JOIN t054_mm_estados estado_co 
                    ON t420.f420_id_grupo_clase_docto = estado_co.f054_id_grupo_clase_docto
                    AND T420.f420_ind_estado = estado_co.f054_id

                WHERE f200_ind_proveedor = 1 AND f200_ind_estado = 1 
                    AND f201_ind_estado_activo = 1 AND f420_ind_estado = 3
                    AND f350_fecha BETWEEN '$dateIn' AND '$dateEnd'
                    
                GROUP BY
                    f420_fecha,
                    f350_fecha,
                    T420.f420_id_tipo_docto,
                    t350_t1.f350_id_tipo_docto,
                    T420.f420_consec_docto,
                    T420.f420_fecha,
                    t350_t1.f350_consec_docto,
                    t350_t1.f350_fecha,
                    T420.f420_ind_estado,
                    PROVEEDOR.f200_id,
                    PROVEEDOR.f200_razon_social,
                    T420.f420_id_sucursal_prov,
                    f201_descripcion_sucursal,
                    T120.f120_referencia,
                    T120.f120_descripcion,
                    T420.f420_id_co,
                    f421_cant_pedida,
                    f421_cant_entrada,
                    f421_precio_unitario,
                    f421_vlr_bruto
            ";
            $arrayResult = Yii::$app->dbsql->createCommand($sql)->queryAll();
        } catch (Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r($exc->getMessage(), true) . '}');
        }
        return $arrayResult;
    }

}
