<?php

namespace app\commands;

use yii\console\Controller;
use yii;
use yii\helpers\ArrayHelper;
use app\components\purchases\ClientSiesa;
use app\models\Fulfillmentprovision;

class FullfillmentProvisionController extends Controller {

    /**
     *
     */
    public function actionIndex() {
        Yii::$app->db->createCommand('TRUNCATE data_fulfillment_provision')->execute();
        $this->nsFullfillmentProvision();
    }

    /**
     * Metodo para extraer e insertar localmente la informacion de siesa 
     * para el calculo del nivel de servicio de aprovisionamiento
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function nsFullfillmentProvision() {
        $dateEnd = date('d-m-Y');
        $dateIn = date('d-m-Y', strtotime('-1 month'));
        $result = $this->_getData($dateIn, $dateEnd);

        if (count($result) > 0) {
            $arrayBlocks = array_chunk($result, 500);
            foreach ($arrayBlocks as $block) {
                $this->_insertData($block);
            }
        }
    }

    /**
     * Metodo para la insercion en bloque de las referencias
     * segun el proveedor
     * 
     * @param array $data
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _insertData($data) {
        try {
            if (empty($data) == false) {
                Yii::$app->db->createCommand()->batchInsert('data_fulfillment_provision',
                        [
                            'idtype',
                            'type',
                            'date',
                            'operation_center',
                            'value_fulfillment_unit',
                            'total_unit',
                            'value_fulfillment_money',
                            'total_money',
                            'value_fulfillment_order',
                            'total_order',
                            'value_fulfillment_references',
                            'total_references'
                           ],
                        $data)->execute();
            }
        } catch (\Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r($exc->getMessage(), true) . '}');
        }
    }

    /**
     * Metodo para obtener la informacion
     * 
     * @param string $dateIn  Fecha Inicial fotmato(dd-mm-YYYY)
     * @param string $dateEnd Fecha Final fotmato(dd-mm-YYYY)
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getData($dateIn, $dateEnd) {
        $arrayResult = array();
        try {
            $sql = "
            SELECT 
                3 as idtype,
                '".Fulfillmentprovision::PENDINGS."' as type,
                f431_fecha_cumplido as date,
                f430_id_co as operation_center,
                SUM(f431_cant1_pedida) AS total_unit,
                -- count(T120.f120_referencia) as referencia,
                SUM(CASE WHEN f431_cant1_facturada > 0 THEN f431_cant1_pedida - f431_cant1_facturada ELSE f431_cant1_pedida - f431_cant1_remisionada END) AS value_fulfillment_unit,
                (SUM((CASE WHEN f431_cant1_facturada > 0 THEN f431_cant1_pedida - f431_cant1_facturada ELSE f431_cant1_pedida - f431_cant1_remisionada END)*(f431_vlr_bruto/f431_cant1_pedida))) AS value_fulfillment_money,
                SUM(f431_vlr_bruto) AS total_money,
                0 AS value_fulfillment_order,
                0 as total_order,
                0 AS value_fulfillment_references,
                0 AS total_references
            FROM
            T430_CM_PV_DOCTO PV 
            
            LEFT JOIN T431_CM_PV_MOVTO MPV
            ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO
            
            LEFT JOIN t121_mc_items_extensiones T121
            ON T121.f121_rowid = MPV.f431_rowid_item_ext

            LEFT JOIN t120_mc_items T120
            ON T120.f120_rowid = T121.f121_rowid_item
            
            Where
                f431_fecha_cumplido BETWEEN '$dateIn' AND '$dateEnd'
                AND PV.f430_ind_estado = 4
            GROUP BY 
             f430_consec_docto, f431_fecha_cumplido, f430_id_co
             
             UNION
             
             -- QUERY VENTAS
            SELECT
                2 as idtype,
                '".Fulfillmentprovision::BILLS."' as type,
                f431_fecha_cumplido as date,
                f430_id_co as operation_center,
                SUM(f431_cant1_pedida) AS total_unit,
                SUM(CASE WHEN f431_cant1_facturada > 0 THEN f431_cant1_facturada ELSE f431_cant1_remisionada END) AS value_fulfillment_unit,
                SUM((CASE WHEN f431_cant1_facturada > 0 THEN f431_cant1_facturada ELSE f431_cant1_remisionada END) *(f431_vlr_bruto/f431_cant1_pedida)) AS value_fulfillment_money,
                SUM(f431_vlr_bruto) AS total_money,
                0 AS value_fulfillment_order,
                0 as total_order,
                0 AS value_fulfillment_references,
                0 AS total_references
            FROM
            T430_CM_PV_DOCTO PV 
            left JOIN T431_CM_PV_MOVTO MPV
            ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

            Where
                f431_fecha_cumplido BETWEEN '$dateIn' AND '$dateEnd'
                AND PV.f430_ind_estado in (3,4)
            GROUP BY 
             f431_fecha_cumplido, f430_id_co
             
             UNION -- --------------------------------------------------------------------------------------

            -- QUERY PEDIDOS
            SELECT
                1 as idtype,
                '".Fulfillmentprovision::ORDERS."' as type,
                f431_fecha_cumplido as date,
                f430_id_co as operation_center,
                SUM(f431_cant1_pedida) AS total_unit,
                SUM(f431_cant1_pedida) AS value_fulfillment_unit,
                SUM(f431_vlr_bruto) AS value_fulfillment_money,
                SUM(f431_vlr_bruto) AS total_money,
                0 AS value_fulfillment_order,
                0 as total_order,
                0 AS value_fulfillment_references,
                0 AS total_references
            FROM
            T430_CM_PV_DOCTO PV 
            left JOIN T431_CM_PV_MOVTO MPV
            	ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO
            Where
                f431_fecha_cumplido BETWEEN '$dateIn' AND '$dateEnd'
                AND PV.f430_ind_estado in (3,4)
            GROUP BY 
             	f431_fecha_cumplido,f430_id_co
             	
            UNION
             	
            SELECT 
                4 as idtype,
                '".Fulfillmentprovision::ORDERS_PENDINGS."' as type,
                f431_fecha_cumplido as date,
                f430_id_co as operation_center,
                0 AS total_unit,
                0 AS value_fulfillment_unit,
                0 AS value_fulfillment_money,
                0 AS total_money,
                SUM(CASE WHEN pedido != facturado OR pedido != remisionado THEN 1 ELSE 0 END) AS value_fulfillment_order,
                COUNT(f430_consec_docto) AS total_order,
                SUM(referencias_incompletas) AS value_fulfillment_references,
                SUM(referencias) AS total_references
                FROM (
                    SELECT
                        f430_consec_docto,
                        f431_fecha_cumplido,
                        f430_id_co,
                        count(f430_consec_docto) as total_pedidos,
                        count(T120.f120_referencia) as referencias,
                        sum(CASE WHEN f431_cant1_pedida != f431_cant1_facturada OR f431_cant1_pedida != f431_cant1_remisionada THEN 1 ELSE 0 END) as referencias_incompletas,
                        sum(f431_cant1_pedida) as pedido,
                        sum(f431_cant1_facturada) as facturado,
                        sum(f431_cant1_remisionada) as remisionado
                    FROM
                    T430_CM_PV_DOCTO PV 
                    
                    left JOIN T431_CM_PV_MOVTO MPV
                    ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                    LEFT JOIN t121_mc_items_extensiones T121
                    ON T121.f121_rowid = MPV.f431_rowid_item_ext

                    LEFT JOIN t120_mc_items T120
                    ON T120.f120_rowid = T121.f121_rowid_item

                    Where
                        f431_fecha_cumplido BETWEEN '$dateIn' AND '$dateEnd'
                        AND PV.f430_ind_estado in (3,4)
                    group by 
                        f430_consec_docto,f431_fecha_cumplido,f430_id_co
            	) AS pnd
            	group by
                    f431_fecha_cumplido,f430_id_co
            		
            	UNION
            	
            	SELECT 
                    5 as idtype,
                    '".Fulfillmentprovision::ORDERS_FULL."' as type,
                    f431_fecha_cumplido as date,
                    f430_id_co as operation_center,
                    0 AS total_unit,
                    0 AS value_fulfillment_unit,
                    0 AS value_fulfillment_money,
                    0 AS total_money,
                    SUM(CASE WHEN pedido = facturado AND pedido = remisionado THEN 1 ELSE 0 END) AS value_fulfillment_order,
                    COUNT(f430_consec_docto) AS total_order,
                    SUM(referencias_incompletas) AS value_fulfillment_references,
                    SUM(referencias) AS total_references
                    FROM (
                        SELECT
                            f430_consec_docto,
                            f431_fecha_cumplido,
                            f430_id_co,
                            count(f430_consec_docto) as total_pedidos,
                            count(T120.f120_referencia) as referencias,
                            sum(CASE WHEN (f431_cant1_pedida = f431_cant1_facturada AND f431_cant1_remisionada = f431_cant1_pedida) THEN 1 ELSE 0 END) as referencias_incompletas,
                            sum(f431_cant1_pedida) as pedido,
                            sum(f431_cant1_facturada) as facturado,
                            SUM(f431_cant1_remisionada) AS remisionado
                        FROM
                        T430_CM_PV_DOCTO PV 
                        left JOIN T431_CM_PV_MOVTO MPV
                        ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                        LEFT JOIN t121_mc_items_extensiones T121
                        ON T121.f121_rowid = MPV.f431_rowid_item_ext

                        LEFT JOIN t120_mc_items T120
                        ON T120.f120_rowid = T121.f121_rowid_item

                        Where
                            f431_fecha_cumplido BETWEEN '$dateIn' AND '$dateEnd'
                            AND PV.f430_ind_estado in (3,4)
                        group by 
                            f430_consec_docto,f431_fecha_cumplido,f430_id_co
            	) AS pnd
            	group by
                    f431_fecha_cumplido,f430_id_co
            		
            UNION
            
            SELECT 
                6 as idtype,
                '".Fulfillmentprovision::ORDERS_TOTAL."' as type,
                f431_fecha_cumplido as date,
                f430_id_co as operation_center,
                0 AS total_unit,
                0 AS value_fulfillment_unit,
                0 AS value_fulfillment_money,
                0 AS total_money,
                COUNT(f430_consec_docto) AS value_fulfillment_order,
                COUNT(f430_consec_docto) AS total_order,
                SUM(referencias) AS value_fulfillment_references,
                SUM(referencias) AS total_references
                FROM (
                    SELECT
                        f430_consec_docto,
                        f431_fecha_cumplido,
                        f430_id_co,
                        count(T120.f120_referencia) as referencias,
                        sum(f431_cant1_pedida) as pedido
                    FROM
                    T430_CM_PV_DOCTO PV 
                    
                    left JOIN T431_CM_PV_MOVTO MPV
                    ON PV.F430_ROWID = MPV.F431_ROWID_PV_DOCTO

                    LEFT JOIN t121_mc_items_extensiones T121
                    ON T121.f121_rowid = MPV.f431_rowid_item_ext

                    LEFT JOIN t120_mc_items T120
                    ON T120.f120_rowid = T121.f121_rowid_item

                    Where
                        f431_fecha_cumplido BETWEEN '$dateIn' AND '$dateEnd'
                        AND PV.f430_ind_estado in (3,4)
                    group by 
                        f430_consec_docto,f431_fecha_cumplido,f430_id_co
            	) AS pnd
            	group by
                    f431_fecha_cumplido, f430_id_co
            ";
            $arrayResult = Yii::$app->dbsql->createCommand($sql)->queryAll();
        } catch (Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r($exc->getMessage(), true) . '}');
        }
        return $arrayResult;
    }

}
