<?php

namespace app\commands;

use yii\console\Controller;
use yii;
use yii\helpers\ArrayHelper;
use app\components\purchases\ClientSiesa;


class ReportInventoryController extends Controller
{
    /**
     *
     */
    public function actionIndex()
    {
        Yii::$app->db->createCommand('TRUNCATE reports_references')->execute();
        $this->_resumenInventario();
    }

    /**
     * Metodo para realizar el prcesamiento del inventario, 
     * realiza el calculo de ventas e inventario, organiza y realiza la
     * insercion en bade de datos
     * 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _resumenInventario() {
        $clientSiesa = new ClientSiesa();
        $bodegas = $clientSiesa->getCellars();

        if (count($bodegas) > 0) {
            $fechaFin = date('t-m-Y', strtotime('-1 month'));            
            $fechaIn = date('01-m-Y', strtotime('-1 month'));            
            $fechaE = date('d-m-Y');
            foreach ($bodegas as $key => $value) {                
                $result = $this->_getInventario($key, $fechaIn, $fechaFin, $fechaE);                
                if(count($result)>0){                    
                    $this->_insertReporteInv($result);                    
                }                                
            }
        }
    }

    /**
     * Metodo para la insercion en bloque de las referencias
     * segun el proveedor
     * 
     * @param array $data
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _insertReporteInv($data) {
        try {
            if(empty($data) == false){
                Yii::$app->db->createCommand()->batchInsert('reports_references', ['reference', 'description', 'cellar_name', 'cellar_id', 'provider_id', 'provider_name', 'branch_office_id', 'branch_office_name',
                'last_cost', 'quantity_sold', 'subtotal', 'margin', 'participation', 'quantity_existence', 'quantity_available', 'quantity_previous', 'quantity_op', 'abc'], $data)->execute();
            }
        } catch (Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . ':' . __FUNCTION__ . ':'
                    . ' {' . print_r($exc->getMessage(), true) . '}'
                    , CLogger::LEVEL_ERROR, 'cmp.db');
        }
    }

    /**
     * Metodo para obtener la informacion de inentario para todos
     * los proveedores segun la bodega y el rango de fecha 
     * 
     * @param int    $idBodega Identificador de la bodega
     * @param string $fechaIn  Fecha Inicial fotmato(dd-mm-YYYY)
     * @param string $fechaFin Fecha Final fotmato(dd-mm-YYYY)
     * 
     * @return array
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2014 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function _getInventario($idBodega, $fechaIn, $fechaFin, $fechaE) {
        $arrayResult = array();
        try {
            $sql = "
            SET NOCOUNT ON;			
            Declare @Nit_Proveedor char(13), @Id_Sucursal char(3) ,@Id_Cia int, @Id_bodega Char(13), @Fecha_I Datetime, @Fecha_F Datetime,@Fecha_E Datetime, 
            
            @Periodo_F char(6), @Total_ventas float, @dias_r integer
            
            set @Nit_proveedor =''
            set @Id_Sucursal = ''
            set @Id_Cia = 1
            Set @Id_bodega = '".  trim($idBodega)."'
            Set @Fecha_I = '".$fechaIn."'
            Set @Fecha_F = '".$fechaFin."'
            Set @Fecha_E = '".$fechaE."'
            
            Set @Total_ventas =  (select sum(q_vtas1.Subtotal_Ventas) from
            (SELECT 
              SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp)*-1 ELSE MFV.f470_vlr_neto-MFV.f470_vlr_imp END)) AS Subtotal_Ventas 
            FROM 
               (T150_MC_BODEGAS T150 RIGHT OUTER JOIN 
                (t470_cm_movto_invent MFV 
                    INNER JOIN t461_cm_docto_factura_venta FV
                    ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto
                    INNER JOIN t350_co_docto_contable DOCFAC
                    ON DOCFAC.f350_rowid = FV.f461_rowid_docto)
                ON T150.F150_ROWID = MFV.F470_ROWID_BODEGA) 
            WHERE 
            mfv.F470_ID_CIA = @Id_Cia
            AND T150.F150_ID = @Id_bodega 
            AND (DOCFAC.f350_fecha BETWEEN @Fecha_I AND @Fecha_F ) 
            AND mfv.F470_IND_TIPO_ITEM = 1
            ) q_vtas1)
            
            SELECT
                T120.f120_referencia as reference,
                T120.f120_descripcion as description, 
                ISNULL(qry.Name_Bodega,' ') as cellar_name,
                ISNULL(qry.Bodega,' ') as cellar_id,
                ISNULL(qry_prov.proveedor,' ') as provider_id,
                ISNULL(qry_prov.Nombre_proveedor,' ') as provider_name,
                ISNULL(qry_prov.Sucursal_proveedor,' ') as branch_office_id,
                ISNULL(qry_prov.Nombre_Sucursal,' ') as branch_office_name,
                ISNULL(T132.f132_ultimo_costo_local,0) AS last_cost,
                CONVERT(INT, ISNULL(SUM(qry.Cantidad_Ventas),0)) as quantity_sold,
                CONVERT(INT, ISNULL(SUM(qry.Subtotal_Ventas),0)) as subtotal,
                ROUND(ISNULL((CASE WHEN SUM(qry.Subtotal_Ventas)= 0 THEN 0 ELSE (SUM(qry.Subtotal_Ventas) - (T132.f132_ultimo_costo_local*SUM(qry.Cantidad_Ventas)))*100/SUM(qry.Subtotal_Ventas) END),0), 2) AS margin, 
                ROUND(ISNULL((CASE WHEN @total_ventas = 0 THEN 0 ELSE CONVERT(DECIMAL(10,1),SUM(qry.Subtotal_Ventas)/@total_ventas*100) END),0), 2) as participation,
                -- ROUND(ISNULL(CONVERT(DECIMAL(10,1),SUM(qry.Subtotal_Ventas)/@total_ventas*100),0), 2) as participation,
                CONVERT(INT, ISNULL((CASE WHEN (SUM(qry.Cantidad_Ventas))=0 THEN 0 ELSE (30*(SUM(qry.Cantidad_Disponible))/(SUM(qry.Cantidad_Ventas))) END),0)) AS quantity_existence,
                CONVERT(INT, ISNULL(SUM(qry.Cantidad_Disponible),0)) As quantity_available,
                CONVERT(INT, ISNULL(SUM(qry.Cantidad_Anterior),0)) As quantity_previous,
                CONVERT(INT, ISNULL(SUM(qry.cantidad_Transito),0)) As quantity_op,
                    T132.f132_abc_rotacion_veces AS abc
            FROM (
                --QUERY VENTAS
                SELECT 
                MFV.f470_rowid_item_ext as RItem,
                T150.f150_descripcion As Name_Bodega,
                T150.F150_ID As Bodega,
                MFV.F470_id_instalacion As Instalacion,
                SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN MFV.f470_cant_1*-1 ELSE MFV.f470_cant_1 END) ) AS Cantidad_Ventas,
                SUM((CASE WHEN MFV.F470_ID_CONCEPTO = '502' THEN (MFV.f470_vlr_neto-MFV.f470_vlr_imp)*-1 ELSE (MFV.f470_vlr_neto-MFV.f470_vlr_imp) END)) AS Subtotal_Ventas,
                0 As Cantidad_Disponible,
                0 As Cantidad_Anterior,
                0 As Cantidad_Transito
            
                FROM 
                   (T150_MC_BODEGAS T150 RIGHT OUTER JOIN
                      (t470_cm_movto_invent MFV 
                        INNER JOIN t461_cm_docto_factura_venta FV
                        ON MFV.f470_rowid_docto_fact = FV.f461_rowid_docto
                        INNER JOIN t350_co_docto_contable DOCFAC
                        ON DOCFAC.f350_rowid = FV.f461_rowid_docto)
                    ON T150.F150_ROWID = MFV.F470_ROWID_BODEGA) 
                WHERE  mfv.F470_ID_CIA = @Id_Cia
                AND T150.F150_ID = @Id_bodega
                    AND T150.f150_ind_estado = 1 
                AND( DOCFAC.f350_fecha BETWEEN @Fecha_I AND @Fecha_F ) 
                AND mfv.F470_IND_TIPO_ITEM = 1
                
                    
                GROUP BY 
                mfv.F470_ROWID_ITEM_EXT, 
                T150.f150_descripcion,
                T150.F150_ID,
                MFV.F470_id_instalacion
            
                UNION 
            
                --QUERY INVENTARIOS Act
                Select
                  T470.F470_ROWID_ITEM_EXT AS RItem,
                  T150.f150_descripcion As Name_Bodega,
                  T150.F150_ID As Bodega,
                      T470.F470_id_instalacion As Instalacion,
                  0 AS Cantidad_Ventas,
                  0 AS Subtotal_Ventas,
                  Sum(CASE WHEN T470.F470_IND_NATURALEZA = 1 THEN T470.F470_CANT_1 ELSE T470.F470_CANT_1*-1 END) As Cantidad_Disponible,
                  0 As Cantidad_Anterior,
                  0 AS Cantidad_Transito
                FROM
                    (T150_MC_BODEGAS T150 Right Outer Join
                       T470_CM_MOVTO_INVENT T470 
                    On T150.F150_ROWID = T470.F470_ROWID_BODEGA)
                WHERE
                  T470.F470_ID_CIA = @Id_CIA
                  AND T150.F150_ID = @Id_bodega
                      AND T150.f150_ind_estado = 1
                  AND T470.F470_ID_fecha <= @Fecha_E 
                  AND T470.F470_IND_ESTADO_CM  <> '2' 
                  AND T470.F470_IND_TIPO_ITEM = 1
                      
                GROUP BY
                  T470.F470_ROWID_ITEM_EXT,
                   T150.f150_descripcion,
                  T150.F150_ID,
                  T470.F470_id_instalacion
            
                UNION
            
                --QUERY INVENTARIOS Ant
                Select
                  T470.F470_ROWID_ITEM_EXT AS RItem,
                  T150.f150_descripcion As Name_Bodega,
                  T150.F150_ID As Bodega,
                  T470.F470_id_instalacion As Instalacion,
                  0 AS Cantidad_Ventas,
                  0 AS Subtotal_Ventas,
                  0 As Cantidad_Disponible,	
                  Sum(CASE WHEN T470.F470_IND_NATURALEZA = 1 THEN T470.F470_CANT_1 ELSE T470.F470_CANT_1*-1 END) As Cantidad_Anterior,
                  0 AS Cantidad_Transito
                From
                  (T150_MC_BODEGAS T150 Right Outer Join
                    T470_CM_MOVTO_INVENT T470 
                    On T150.F150_ROWID = T470.F470_ROWID_BODEGA) 
                Where
                  T470.F470_ID_CIA = @Id_CIA
                  AND T150.F150_ID = @Id_bodega
                      AND T150.f150_ind_estado = 1
                  AND T470.F470_ID_fecha <= @Fecha_I 
                  AND T470.F470_IND_ESTADO_CM  <> '2' 
                  AND T470.F470_IND_TIPO_ITEM = 1
                  
                Group By
                  T470.F470_ROWID_ITEM_EXT,
                   T150.f150_descripcion,
                  T150.F150_ID,
                  T470.F470_id_instalacion
            
                UNION
                --QUERY ORDEN DE COMPRA
                SELECT 
                T421.f421_rowid_item_ext as RItem,
                T150.f150_descripcion As Name_Bodega,
                T150.f150_id AS Bodega,
                T150.F150_id_instalacion As Instalacion,
                  0 AS Cantidad_Ventas,
                  0 AS Subtotal_Ventas,
                  0 As Cantidad_Disponible,
                  0 As Cantidad_Anterior,
                  SUM(T421.f421_cant_pedida) AS Cantidad_Transito
            
                FROM 
                    t054_mm_estados C054 
                    RIGHT OUTER JOIN (t150_mc_bodegas T150 
                        RIGHT OUTER JOIN (t420_cm_oc_docto T420 
                            INNER JOIN t421_cm_oc_movto T421 
                            ON T420.f420_id_cia = T421.f421_id_cia
                            AND T420.f420_rowid = T421.f421_rowid_oc_docto)
                        ON T150.f150_rowid = T421.f421_rowid_bodega)
                    ON ( ( C054.f054_id = T420.f420_ind_estado and c054.f054_id_grupo_clase_docto= t420.f420_id_grupo_clase_docto) ) 
                WHERE T420.f420_id_cia = @Id_Cia 
                AND T150.f150_id = @Id_bodega
                    AND T150.f150_ind_estado = 1
                AND( T420.f420_fecha <= @Fecha_E )
                AND T420.f420_ind_estado IN(1,2)
                
                GROUP BY T421.f421_rowid_item_ext,
                T150.f150_descripcion,
                T150.f150_id,
                T150.F150_id_instalacion
                ) qry
            
                LEFT JOIN T132_Mc_items_instalacion T132 
                On qry.ritem = T132.f132_rowid_item_ext
                and qry.instalacion = T132.f132_id_instalacion 
                and T132.f132_id_cia =  @Id_CIA
                and T132.f132_id_instalacion is not null
                
                FULL JOIN t121_mc_items_extensiones T121
                On qry.ritem = T121.f121_rowid
                    AND f121_ind_estado = 1
                
                INNER JOIN t120_mc_items T120
                On T121.f121_rowid_item = T120.f120_rowid
                AND T120.F120_IND_TIPO_ITEM = '1'
                  
                --QUERY PROVEEDORES 
                LEFT JOIN (
                SELECT 
                    t470_comp.f470_rowid_item_ext AS RItem,
                    T200.f200_id AS Proveedor,
                    T200.f200_razon_social AS Nombre_Proveedor,
                    dtocmp.f451_id_sucursal_prov AS Sucursal_proveedor,
                    T202.f202_descripcion_sucursal as Nombre_Sucursal,
                    RANK() OVER (PARTITION BY t470_comp.f470_rowid_item_ext ORDER BY SUM(CASE WHEN t470_comp.f470_ind_naturaleza = 2 THEN (-t470_comp.f470_cant_1) ELSE t470_comp.f470_cant_1 END) DESC) as Rango
                 FROM t470_cm_movto_invent t470_comp
                    INNER JOIN t451_cm_docto_compras dtocmp ON dtocmp.f451_rowid_docto = t470_comp.f470_rowid_docto
                    INNER JOIN t350_co_docto_contable t350_t1 ON t350_t1.f350_rowid = t470_comp.f470_rowid_docto
                    LEFT JOIN t200_mm_terceros T200 ON dtocmp.f451_rowid_tercero_prov = T200.f200_rowid
                    LEFT JOIN t202_mm_proveedores T202 ON dtocmp.f451_id_sucursal_prov = T202.f202_id_sucursal AND T200.f200_rowid = T202.f202_rowid_tercero
                 WHERE 
                    t350_t1.f350_id_clase_docto IN (408, 409, 410, 411, 412, 413, 414, 417, 420)
                    AND (t470_comp.f470_id_fecha BETWEEN DATEADD(month, -6,@Fecha_F) AND @Fecha_F )
                    --AND T200.f200_id = @Nit_proveedor
                    --AND dtocmp.f451_id_sucursal_prov = @id_sucursal 
                 GROUP BY 
                    t470_comp.f470_rowid_item_ext, 
                    T200.f200_id,
                    T200.f200_razon_social,
                    T202.f202_descripcion_sucursal,
                    dtocmp.f451_id_sucursal_prov) qry_prov
                ON qry.ritem = qry_prov.ritem
                AND qry_prov.RANGO = 1
                    
                    WHERE qry.Bodega IS NOT NULL AND qry_prov.proveedor IS NOT NULL
                
            GROUP BY 
            T120.f120_referencia,
            T120.f120_id,
            T120.f120_descripcion, 
            T120.F120_IND_TIPO_ITEM,
            qry.Name_Bodega,
            qry.Bodega,
            qry_prov.proveedor,
            qry_prov.Nombre_proveedor,
            qry_prov.Sucursal_proveedor,
            qry_prov.Nombre_Sucursal,
            T132.f132_ultimo_costo_local,
            T132.f132_abc_rotacion_veces
            
            ORDER BY T120.f120_referencia, T120.f120_id,qry.Bodega,qry_prov.proveedor,qry_prov.Nombre_proveedor,qry_prov.Sucursal_proveedor
        ";
            $arrayResult = Yii::$app->dbsql->createCommand($sql)->queryAll();
        } catch (Exception $exc) {
            //----------------------------------------------------
            \Yii::error(__FILE__ . ':' . __LINE__ . ':' . __FUNCTION__ . ':'
                    . ' {' . $exc->getMessage() . '}', CLogger::LEVEL_ERROR, 'comp.bd');
            //---------------------------------------------------            
        }
        return $arrayResult;
    }

}
