<?php
namespace app\components;

use Yii;
use yii\base\Component;

/**
 * WebServices.php
 *
 * PHP Version 5.3.0
 * 
 * @category  category
 * @package   package
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2013 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version   SVN: $Id$
 * @link      http://www.ingeneo.com.co
 * 
 */

/**
 * WebServices.php
 *
 * // SUS COMENTARIOS AQUI
 *
 * @category  category
 * @package   package
 * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
 * @copyright 2013 INGENEO S.A.S.
 * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version   Release: $Id$
 * @link      http://www.ingeneo.com.co
 * 
 */
class WebServices extends Component {

    /**
     * Parametro de tipo WebServices
     * @var WebServices
     */
    public $webServices;
    
    /**
     * Parametro con la respuesta del servicio
     * @var string
     */
    public $response;
    
    /**
     * Constructor de la clase
     * 
     * @return void
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version   SVN: $Id$
     * @copyright 2013 INGENEO S.A.S.     
     * @access public     
     */
    public function __construct()
    {
        
    }

    /**
     * Funcion desctructora 
     * 
     * @return void
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @version   SVN: $Id$
     * @copyright 2013 INGENEO S.A.S. 
     * @access public     
     */
    public function __destruct()
    {
        
    }

    /**
     * Metodo para validar la activacion del validateUrlWebServices
     * 
     * @return boolean 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2013 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    private function isActiveValidateUrl()
    {
        $codeActive = \Yii::$app->params['validUrl'];
        if(!$codeActive) {
            return false;
        }
        return true;
    }

    /**
     * Metodo para validar si el contenido de un string es xml
     * 
     * @param string $string Cadena xml a validar
     * 
     * @return boolean 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2013 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function validateStringXml($string)
    {
        try {
            $content = file_get_contents($string);
            if($content && !empty($content)) {
                $resulXml = simplexml_load_string($content);
                if($resulXml) {                    
                    \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r(' {validateStringXml [' . $resulXml 
                    . '}', true) . '}', 'webservices');
                    return true;
                }
            }
        } catch(\Exception $exc) {            
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r(' {validateStringXml [' . $exc->getMessage() 
                    . '}', true) . '}', 'webservices');
            return false;
        }
        return false;
    }

    /**
     * Metodo para la validacion de servicios web
     * 
     * @param string $url Url del servicio
     * 
     * @return boolean 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2013 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    function validateUrlWebServices($url)
    {
        if(!$this->isActiveValidateUrl()) {
            return true;
        }

        // Primero, compruebo si la URL es una URL valida
        if(!filter_var($url, FILTER_VALIDATE_URL)) {
            return false;
        }
        // Inicializo y configuro una peticion con CURL
        $curlInit = curl_init($url);

        // Limito el tiempo de espera en 5 segundos y configuro las opciones
        curl_setopt($curlInit, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt($curlInit, CURLOPT_HEADER, true);
        curl_setopt($curlInit, CURLOPT_NOBODY, true);
        curl_setopt($curlInit, CURLOPT_RETURNTRANSFER, true);

        // Obtengo una respuesta
        $response = curl_exec($curlInit);
        curl_close($curlInit);

        if($response) {
            if($this->validateStringXml($url)) {                
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                        . print_r('{url correcta validateUrlWebServices [' 
                                . $url . ']}', true) . '}', 'webservices');
                return true;
            }
        }                
        \Yii::error(__FILE__ . ':' . __LINE__ . '{'
            . print_r('{url incorrecta validateUrlWebServices [' 
            . $url . ']}', true) . '}', 'webservices');
        return false;
    }

    /**
     * Metodo para instanciar el servicio web
     * 
     * @param string $url      Url del servicio
     * @param string $login    Login del servicio (Opcional)
     * @param string $password Password del servicio (Opcional)
     * 
     * @return boolean 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2013 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function webServices($url, $login = NULL, $password = NULL)
    {
        $validateWs = $this->validateUrlWebServices($url);
        if($validateWs) {
            try {
                $socketTime = 1;
                if(!is_null($login) && !is_null($password)) {
                    $this->webServices = new \SoapClient(
                        $url,
                        array(
                        'login' => $login,
                        'password' => $password,
                        'cache_wsdl' => WSDL_CACHE_DISK,
                        'default_socket_timeout' => $socketTime,
                        )
                    );
                } else {
                    $this->webServices = new \SoapClient($url,
                        array(
                        'cache_wsdl' => WSDL_CACHE_DISK,
                        'default_socket_timeout' => $socketTime,
                    ));
                }                
                return true;
            } catch(\SoapFault $exc) {                
                \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r(' {[' . $exc->getMessage() 
                    . '}', true) . '}', 'webservices');
                return false;
            }
        }
        return false;
    }

    /**
     * Metodo para ejecutar metodos del servicio
     * 
     * @param string $method   Nombre del metodo
     * @param string $url      Url del servicio
     * @param array  $params   Parametros del metodo
     * @param string $login    Login de autenticacion del servicio
     * @param string $password Clave de autenticacion del servicio
     * 
     * @return boolean 
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2013 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co
     */
    public function executeMethodWs($method, $url, $params = null, $login = null, $password = null)
    {
        try {
            if($this->webServices($url, $login, $password)) {
                if(is_string($method) && !empty($method)) {
                    try {
                        if(is_null($params)) {
                            $result = $this->webServices->$method();
                        } else {
                            $result = $this->webServices->$method($params);
                        }

                        // \Yii::error(__FILE__ . ':' . __LINE__ . '{'. print_r($result, true) . '}', 'webservices');

                        if($result) {
                            $this->response = $result;                            
                            return true;
                        }

                        \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                            . print_r(' {executeMethodWs [' . $result 
                            . '}', true) . '}', 'webservices');

                        return false;
                    } catch(\SoapFault $exc) {                        
                        \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                            . print_r(' {executeMethodWs [' . $exc->getMessage() 
                            . '}', true) . '}', 'webservices');
                        return false;
                    }
                }
            }
        } catch(\SoapFault $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r(' {executeMethodWs [' . $exc->getMessage() 
                    . '}', true) . '}', 'webservices');
            return false;
        } catch(\Exception $exc) {
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r(' {executeMethodWs [' . $exc->getMessage() 
                    . '}', true) . '}', 'webservices');
            return false;
        }

        return false;
    }

}

?>
