<?php

namespace app\controllers;

use Yii;
use app\models\Fulfillmentprovision;
use app\models\FulfillmentprovisionSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\components\HelperReport;
use yii\helpers\HtmlPurifier;

/**
 * FulfillmentprovisionController implements the CRUD actions for Fulfillmentprovision model.
 */
class FulfillmentprovisionController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Fulfillmentprovision models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new FulfillmentprovisionSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Fulfillmentprovision model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Fulfillmentprovision model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Fulfillmentprovision();
        $model->scenario = 'preview';
        $preview = true;
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        $valueListDefault = $model->listParamsDefault();
        $valueListAdd = $model->listParamsAdd();                
        if ($model->load(Yii::$app->request->post())) {            
            if(isset($_POST['paramsAdd']) && isset($_POST['paramsDefault']) 
                    && !empty($_POST['paramsDefault'])){
                $valueListAdd = HtmlPurifier::process($_POST['paramsAdd']);
                $valueListDefault = HtmlPurifier::process($_POST['paramsDefault']);
            }
            # Si se crea el reporte --------------------------------------------
            if(isset($_POST['create'])){                
                $model->scenario = 'create'; 
                $model->paramsAdd = $valueListAdd;
                $model->paramsDefault = $valueListDefault;
                if ($model->load(Yii::$app->request->post()) && $model->validate()) {
                    $preview = false;
                    if($model->save()){
                        return $this->redirect(['update', 'id' => $model->id]);
                    }
                }
            }                                    
        }
        
        $helperReport = new HelperReport;
        $listParamsAdd = $helperReport->getTempList($valueListAdd, $model->listParams());
        $listParamsDefault = $helperReport->getTempList($valueListDefault, $model->listParams());
        $columns =  $helperReport->getTempList($valueListDefault, $model->listParamsGrid()); 
        $dataProvider = $model->reportFulfillment($valueListDefault, $queryParams, $preview); 
        
        return $this->render('create', [
            'model' => $model,
            'listParamsDefault' => $listParamsDefault,
            'listParamsAdd' => $listParamsAdd,
            'valueListAdd' => $valueListAdd,
            'valueListDefault' => $valueListDefault,
            'columns' => $columns,
            'dataProvider' => $dataProvider,
            'preview' => $preview
        ]); 
    }

    /**
     * Updates an existing Fulfillmentprovision model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        $helperReport = new HelperReport;
        
        $model->scenario = 'preview';
        $preview = false;
        # Obtenemos los parametros de la busqueda ------------------------------
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        # ---------------------------------------------------------------------
        $valueListDefault = $model->paramsDefault;            
        $valueListAdd = $model->paramsAdd;
        # Tomamos el POST ------------------------------------------------------
        if ($model->load(Yii::$app->request->post())) {            
            if(isset($_POST['paramsAdd']) && isset($_POST['paramsDefault']) 
                    && !empty($_POST['paramsDefault'])){
                $valueListAdd = HtmlPurifier::process($_POST['paramsAdd']);
                $valueListDefault = HtmlPurifier::process($_POST['paramsDefault']);
            }
            # Si se crea el reporte --------------------------------------------
            if(isset($_POST['create'])){                
                $model->scenario = 'create';                
                if ($model->load(Yii::$app->request->post()) && $model->validate()) {
                    $model->paramsAdd = $valueListAdd;
                    $model->paramsDefault = $valueListDefault;                    
                    $model->save();
                    return $this->redirect(['update', 'id' => $model->id]);
                }
            }
            
            if(isset($_POST['preview'])){ 
                $preview = true;
            }
        }        
       
        $listParamsAdd = $helperReport->getTempList($valueListAdd, $model->listParams());
        $listParamsDefault = $helperReport->getTempList($valueListDefault, $model->listParams());
        $columns =  $helperReport->getTempList($valueListDefault, $model->listParamsGrid()); 
        $dataProvider = $model->reportFulfillment($valueListDefault, $queryParams, $preview);
             
        if(isset($_REQUEST['exportar'])){                    
            return $this->render('_exportGrid', [
                'model' => $model,
                'listParamsDefault' => $listParamsDefault,
                'listParamsAdd' => $listParamsAdd,
                'valueListAdd' => $valueListAdd,
                'valueListDefault' => $valueListDefault,
                'columns' => $columns,
                'dataProvider' => $dataProvider,
                'preview' => $preview
            ]);
        }else{
            return $this->render('update', [
                'model' => $model,
                'listParamsDefault' => $listParamsDefault,
                'listParamsAdd' => $listParamsAdd,
                'valueListAdd' => $valueListAdd,
                'valueListDefault' => $valueListDefault,
                'columns' => $columns,
                'dataProvider' => $dataProvider,
                'preview' => $preview
            ]);
        }
    }

    /**
     * Deletes an existing Fulfillmentprovision model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Fulfillmentprovision model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Fulfillmentprovision the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Fulfillmentprovision::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }
    
    /**
     * 
     * @return type
     */
    public function actionGetdatagraficas(){
        if (isset($_POST['dataEjex']) && !empty($_POST['dataEjex']) 
                && !empty($_POST['dataEjey']) ) {
            $request_url = ($_SERVER["HTTP_REFERER"]);
            $get = parse_url($request_url);
            parse_str($get['query'], $queryUrl);            
            $queryParams = array_merge(array(), $queryUrl);          
            # Limpiamos los datos  ---------------------------------------------   
            $dataEjex = HtmlPurifier::process(implode(',',$_POST['dataEjex']));           
            $dataEjey = HtmlPurifier::process(implode(',',$_POST['dataEjey']));           
            $type = HtmlPurifier::process($_POST['type']);  
            $id = HtmlPurifier::process($_POST['id']);  
            if(!empty($id)){                                
                $model = $this->findModel($id);
                $model->ejex = $dataEjex;
                $model->ejey = $dataEjey;
                $model->type = $type;
                $model->save();
            }
            #-------------------------------------------------------------------
            $model = new Fulfillmentprovision;
            return $model->reportGraph($dataEjex, $dataEjey, $queryParams, $type);                                
        }
    }
}
