<?php

namespace app\controllers;

use Yii;
use app\models\Items;
use app\models\Purchases;
use app\models\PurchasesSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\web\Response;
use app\components\purchases\ClientSiesa;
use yii\helpers\Json;
use yii\widgets\ActiveForm;
use yii\helpers\HtmlPurifier;
use app\models\Itemspurchase;
use app\components\WebServices;
use app\components\purchases\FactoryOrder;

/**
 * PurchasesController implements the CRUD actions for Purchases model.
 */
class PurchasesController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                    'create' => ['POST', 'GET'],
                ],
            ],
        ];
    }

    /**
     * Lists all Purchases models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new PurchasesSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Purchases model.
     * @param string $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Purchases model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Purchases();
        
        if (Yii::$app->request->post()) {
            if(isset($_POST['chAsk'])){
                $error = false;
                $errorData = false;
                $postData = Yii::$app->request->post('chAsk');
                $arrrayQuant = Yii::$app->request->post('item');
                $arrayUnitValues = Yii::$app->request->post('unit_value');
                $arrayDesc = Yii::$app->request->post('item_desc');
				$arrayMu = Yii::$app->request->post('measured_unit');
                $summaryError = '';
                if (count($postData)) {
                    $arrayData = array();
                    $model->load(Yii::$app->request->post());
                    $model->save();
                    $total = 0;
                    foreach ($postData as $key => $value) {
                        $reference = $key;
                        $description = (isset($arrayDesc[$key])) ? $arrayDesc[$key] : '';
                        $quantity = (isset($arrrayQuant[$key])) ? $arrrayQuant[$key] : '';
                        $unitValue = (isset($arrayUnitValues[$key])) ? $arrayUnitValues[$key] : '';
						$measuredUnit = (isset($arrayMu[$key])) ? $arrayMu[$key] : Items::UNID;
                        if (!empty($reference) && is_numeric($reference) 
                                && !empty($quantity) && is_numeric($quantity) 
                                && !empty($unitValue) && is_numeric($unitValue)) {
									
							if(trim($measuredUnit) != Items::UNID){                
								$undMed = preg_replace('/[^0-9]+/', '', $measuredUnit);                                        
								if($undMed > 0){                       
									$quantity = $quantity * $undMed;
								}                                        
							}
									
                            $itemPurchase = new Itemspurchase();
                            $itemPurchase->reference = (string)$reference;
                            $itemPurchase->description = $description;
                            $itemPurchase->quantity = $quantity;
                            $itemPurchase->unit_price = $unitValue;
                            $itemPurchase->purchase_id = $model->id;
                            $itemPurchase->save();
                            $total = $total + ($quantity * $unitValue);
                        } else {
                            $summaryError .= ' Referecia: ' . $reference
                                    . ' Cantidad: ' . $quantity
                                    . ' Precio Un: ' . $unitValue
                                    . '<br/>';
                            $errorData = true;
                        }
                    }
                    if (!$errorData) { 
                        $model->total = $total;
                        $model->save();
                        $error = ($this->_saveOrderSiesa($model))? false : true;
                    }else{
                        $model->delete();
                    }
                } else {
                    $error = true;
                }

                if ($errorData) {
                    Yii::$app->session->setFlash('danger', 
                            '<strong>Alerta!</strong> Datos Incorrectos: <br/>'
                            . $summaryError);
                }

                if ($error) {
                    $model->delete();
                    Yii::$app->session->setFlash('danger',Yii::t('app', 'error send order'));
                } else {
                    Yii::$app->session->setFlash(
                            "success", Yii::t('app', 'success send order'));
                    return $this->redirect(['create']);
                }
            }
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Finds the Purchases model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param string $id
     * @return Purchases the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Purchases::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException(Yii::t('app', 'The requested page does not exist.'));
    }
    
    /**
     * Retorna el cliente buscado
     * 
     * @param type $q
     * @param type $id
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetlistproviders($q = null, $id = null) {
        \Yii::$app->response->format = Response::FORMAT_JSON;        
        $data = [];
        if (!is_null($q)) {
            $data = Purchases::selectProviders($q);                                    
        }
        elseif ($id > 0) {
            ## Buscamos en todos los ERP ---------------------------------------                                      
//            $out['results'] = ['id' => $id, 'text' => City::find($id)->name];
        }
        $out['results'] = $data;
        return $out;
    }
    
    /**
     * Retorna el cliente buscado
     * 
     * @param type $q
     * @param type $id
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2018 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetlistbranchs($q = null, $id = null) {
        $out = [];
        if (isset($_POST['depdrop_all_params']['purchases-nit'])) {
            $nit = $_POST['depdrop_all_params']['purchases-nit'];            
            if ($nit != null) {
                $out = Purchases::selectBranches($nit);                               
                echo Json::encode(['output'=>$out, 'selected'=>'']);
                return;
            }
        }
        echo Json::encode(['output'=>'', 'selected'=>'']);
    }
    
    public function actionValidateform() {
        $model = new Purchases();
        if (Yii::$app->request->isAjax && $model->load(Yii::$app->request->post())) {
            \Yii::$app->response->format = Response::FORMAT_JSON;
            return ActiveForm::validate($model);
        }
    }
    
    /**
     * Retorna el listado de productos 
     * 
     * @return json
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2019 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    public function actionGetitems() {       
        $model = new Purchases();
        if (Yii::$app->request->isAjax && $model->load(Yii::$app->request->post())) {
            $withPage = HtmlPurifier::process($_POST['withPage']);
            $listProduct = $model->getItems();
            return $this->renderAjax('_formProducts', [
                        'model' => $model,
                        'listProduct' => $listProduct,
                        'withPage' => $withPage,
            ]); 
        }     
    }
    
    /**
     * Envia el pedido a los diferente ERP
     * 
     * @param Pedidos $model
     * 
     * @return boolean
     * @author Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license http://www.opensource.org/licenses/bsd-license.php  BSD License
     * @version SVN: $Id$
     * @link http://www.ingeneo.com.co
     */
    private function _saveOrderSiesa($model){
        $envioSiesa = false;
        try {
            if (!empty($model) && is_object($model)) {
                $factoryOrder = new FactoryOrder;
                $xml = $factoryOrder->createOrder($model);
                if (!empty($xml)) {
                    $webServices = new WebServices();
                    $method = 'ImportarXML';
                    $url = \Yii::$app->params['paramsWs']['urlWs'];
                    $params = array('pvstrDatos' => $xml, 'printTipoError' => 0);
                    if ($webServices->executeMethodWs($method, $url, $params)) {
                        $response = $webServices->response;
                        if (isset($response->printTipoError) && $response->printTipoError === 0) {                                    
                            $envioSiesa = true;
                        } else {
                            $error = $response->ImportarXMLResult->any;                                  
                            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                                    . print_r('{Error xml ['
                                            . $xml . ']}', true) . '}',
                                    'webservices');
                            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                                    . print_r('{Error xml ['
                                            . $error . ']}', true) . '}',
                                    'webservices');
                            $envioSiesa = false;
                        }
                    }
                }
            }            
        } catch (\Exception $exc) {
            $envioSiesa = false;
            \Yii::error(__FILE__ . ':' . __LINE__ . '{'
                    . print_r('{Error ['. $exc->getMessage() 
                    . ']}', true) . '}', 'webservices');
        } 
        return $envioSiesa;
    }
}
