<?php

namespace app\controllers;

use Yii;
use app\models\Reports;
use app\models\ReportsDynamic;
use app\models\ReportsSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\helpers\HtmlPurifier;

/**
 * ReportsController implements the CRUD actions for Reports model.
 */
class ReportsController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Reports models.
     * @return mixed
     */
    public function actionIndex()
    {
        $session = Yii::$app->session;
        $session->destroySession('addParameters');
        $session->destroySession('defaultParameters');
        $searchModel = new ReportsSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Reports model.
     * @param integer $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Reports model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Reports();
        $reportDynamic = new ReportsDynamic();
        $preview = true;
        $session = Yii::$app->session;
        $session->destroySession('addParameters');
        $session->destroySession('defaultParameters');

        //Obtenemos los parametros de la búsqueda ------------------------------
        $queryParameters = array_merge(array(), Yii::$app->request->getQueryParams());
        // ---------------------------------------------------------------------
        $valueListDefaultParameters = $reportDynamic->listDefaultParameters();
        $valueListParametersAdd = $reportDynamic->listParametersAdd();


        # ----------------------------------------------------------------------
        if ($model->load(Yii::$app->request->post())) {
            if(isset($_POST['addParameters']) && isset($_POST['defaultParameters'])){
                $valueListAddParameters = HtmlPurifier::process($_POST['addParameters']);
                $valueListdefaultParameters = HtmlPurifier::process($_POST['defaultParameters']);
            }
            $model->user_id = \Yii::$app->user->getId();
            if ($model->load(Yii::$app->request->post()) && $model->validate()) {
                $model->addParameters = $valueListAddParameters;
                $model->defaultParameters = $valueListdefaultParameters;
                $preview = false;
                if($model->save()){
                    return $this->redirect(['update', 'id' => $model->id]);
                }
            }
        }

        # Sacamos el listado de los parametros ---------------------------------
        $listParametersAdd = $reportDynamic->getTempList($valueListParametersAdd);
        $listDefaultParameters = $reportDynamic->getTempList($valueListDefaultParameters);
        $columns =  $reportDynamic->getListGrid($valueListDefaultParameters);

        return $this->render('create', [
            'model' => $model,
            'reportDynamic' => $reportDynamic,
            'listDefaultParameters' => $listDefaultParameters,
            'listParametersAdd' => $listParametersAdd,
            'valueListParametersAdd' => $valueListParametersAdd,
            'valueListDefaultParameters' => $valueListDefaultParameters,
            'preview' => $preview,
            'columns' => $columns,
        ]);        
    }

    /**
     * Updates an existing Reports model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $session = Yii::$app->session;
        $session->destroySession('addParameters');
        $session->destroySession('defaultParameters');
        $model = $this->findModel($id);
        $searchModel = new ReportsDynamic();
        $preview = false;
        # Obtenemos los parametros de la busqueda ------------------------------
        $queryParams = array_merge(array(), Yii::$app->request->getQueryParams());
        # ---------------------------------------------------------------------
        $valueListDefaultParameters = $model->defaultParameters;
        $valueListParametersAdd = $model->addParameters;

        # Tomamos el POST ------------------------------------------------------
        if ($model->load(Yii::$app->request->post())) {
            if(isset($_POST['addParameters']) && isset($_POST['defaultParameters'])){
                $valueListAddParameters = HtmlPurifier::process($_POST['addParameters']);
                $valueListdefaultParameters = HtmlPurifier::process($_POST['defaultParameters']);
            }
            $model->user_id = \Yii::$app->user->getId();
            if ($model->load(Yii::$app->request->post()) && $model->validate()) {
                $model->addParameters = $valueListAddParameters;
                $model->defaultParameters = $valueListdefaultParameters;
                $preview = false;
                if($model->save()){
                    return $this->redirect(['update', 'id' => $model->id]);
                }
            }
        }

        //Sacamos el listado de los parametros ---------------------------------
        $listParametersAdd = $searchModel->getTempList($valueListParametersAdd);
        $listDefaultParameters = $searchModel->getTempList($valueListDefaultParameters);
        $columns =  $searchModel->getListGrid($valueListDefaultParameters);
        $dataProvider = $searchModel->report($valueListDefaultParameters, $queryParams, $preview);
             
        if(isset($_REQUEST['exportar'])){                    
            return $this->render('_exportGrid', [
                'model' => $model,
                'searchModel' => $searchModel,
                'listDefaultParameters' => $listDefaultParameters,
                'listParametersAdd' => $listParametersAdd,
                'valueListParametersAdd' => $valueListParametersAdd,
                'valueListDefaultParameters' => $valueListDefaultParameters,
                'columns' => $columns,
                'dataProvider' => $dataProvider,
                'preview' => $preview
            ]);
        }else{
            return $this->render('update', [
                'model' => $model,
                'searchModel' => $searchModel,
                'listDefaultParameters' => $listDefaultParameters,
                'listParametersAdd' => $listParametersAdd,
                'valueListParametersAdd' => $valueListParametersAdd,
                'valueListDefaultParameters' => $valueListDefaultParameters,
                'preview' => $preview,
                'columns' => $columns,
                'dataProvider' => $dataProvider,
            ]);
        }        
    }

    /**
     * Deletes an existing Reports model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Reports model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Reports the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Reports::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }

    /**
     * 
     * @return type
     */
    public function actionGetDataGraph(){
        if (isset($_POST['dataEjex']) && !empty($_POST['dataEjex']) 
                && !empty($_POST['dataEjey']) ) {
            $request_url = ($_SERVER["HTTP_REFERER"]);
            $get = parse_url($request_url);
            parse_str($get['query'], $queryUrl);            
            $queryParams = array_merge(array(), $queryUrl);          
            # Limpiamos los datos  ---------------------------------------------   
            $dataEjex = HtmlPurifier::process(implode(',',$_POST['dataEjex']));           
            $dataEjey = HtmlPurifier::process(implode(',',$_POST['dataEjey']));           
            $type = HtmlPurifier::process($_POST['type']);
            $id = HtmlPurifier::process($_POST['id']);  
            if(!empty($id)){                                
                $model = $this->findModel($id);
                $model->ejex = $dataEjex;
                $model->ejey = $dataEjey;
                $model->type = $type;
                $model->save();
            }
            #-------------------------------------------------------------------
            $model = new ReportsDynamic;
            return $model->reportGraph($dataEjex, $dataEjey, $queryParams, $type);
        }
    }


    /**
     * 
     * @return type
     */
    public function actionDashboard(){
        $model = new ReportsDynamic;
        $cargo = $model->cargoMayorAusentismo();
        $empleado = $model->empleadoMayorAusentismo();
        $centroCostos = $model->centroCostosMayorAusentismo();
        $linea = $model->lineaMayorAusentismo();
        $info = $model->getInfoDasboard();        
        return $this->render('dashboard', [
            'model' => $model,            
            'cargo' => $cargo,            
            'empleado' => $empleado,            
            'centroCostos' => $centroCostos,            
            'linea' => $linea,            
            'info' => $info,            
        ]);
    }

    public function actionGetInventories(){
        $post = Yii::$app->request->post();
        $get = Yii::$app->request->get();
        $session = Yii::$app->session;
        $model = new Reports();
        $searchModel = new ReportsDynamic();
        $queryParameters = array_merge(array(), Yii::$app->request->getQueryParams());
        $valueListParametersAdd = '';
        $valueListDefaultParameters = '';

        if(empty($post) == FALSE && empty($post['addParameters']) == FALSE && empty($post['defaultParameters']) == FALSE) {
            $valueListParametersAdd = $post['addParameters'];
            $valueListDefaultParameters = $post['defaultParameters'];
            $session->set('addParameters', $post['addParameters']);
            $session->set('defaultParameters', $post['defaultParameters']);
        }else if($session->has('addParameters') == TRUE && $session->has('defaultParameters') == TRUE){
            $valueListParametersAdd = $session->get('addParameters');
            $valueListDefaultParameters = $session->get('defaultParameters');
        }

        if(empty($valueListParametersAdd) == FALSE && empty($valueListDefaultParameters) == FALSE){
            $preview = false;
            //Sacamos el listado de los parametros ---------------------------------
            $searchModel->getTempList($valueListParametersAdd);
            $listDefaultParameters = $searchModel->getTempList($valueListDefaultParameters);
            $columns = $searchModel->getListGrid($valueListDefaultParameters);
            $dataProvider = $searchModel->report($valueListDefaultParameters, $queryParameters, $preview);

            return $this->renderAjax('_inventories', [
                'model' => $model,
                'dataProvider' => $dataProvider,
                'preview' => $preview,
                'columns' => $columns,
                'btnRefress' => null,
                'searchModel' => $searchModel,
                'valueListParametersAdd' => $valueListParametersAdd,
                'valueListDefaultParameters' => $valueListDefaultParameters,
                'listDefaultParameters' => $listDefaultParameters,
            ]);
        }

        \Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        return [
            'status' => 'ERROR',
        ];
    }

}