<?php

namespace app\models;
use app\models\Cmpuser;
use app\components\RijndaelHex;
use yii\db\mssql\PDO;

class User extends \yii\base\BaseObject implements \yii\web\IdentityInterface
{
    public $id;
    public $username;
    public $password;
    public $authKey;
    public $accessToken;
    public $fullName;
    public $identificacion;

    private static $users = [
    ];


    /**
     * {@inheritdoc}
     */
    public static function findIdentity($id) {
        $fnIdentity = \Yii::$app->session['fnIdentity'];
        if (empty($fnIdentity)) {
            $sql = "SELECT usr_user_ID as id, usr_username as username, "
                . "usr_username as authKey, usr_username as accessToken, "
                . "usr_username as fullName, usr_password as password, "
                . "usr_cedula as identificacion "
                . "FROM cmp_user u "
                . "WHERE usr_user_ID = :id";
            $command = \Yii::$app->db->createCommand($sql);
            $command->bindParam(":id", $id, PDO::PARAM_STR);
            $user = $command->queryOne();
            \Yii::$app->session['fnIdentity'] = $user;
        } else {
            $user = \Yii::$app->session['fnIdentity'];
            \Yii::$app->session->destroy();
        }
        if ($user) {
            return new static($user);
        }
        return null;
    }

    /**
     * {@inheritdoc}
     */
    public static function findIdentityByAccessToken($token, $type = null) {
        foreach (self::$users as $user) {
            if ($user['accessToken'] === $token) {
                return new static($user);
            }
        }

        return null;
    }

    /**
     * Finds user by username
     *
     * @param string $username
     * @return static|null
     */
    public static function findByUsername($username) {
        $sql = "SELECT usr_user_ID as id, usr_username as username, "
                . "usr_username as authKey, usr_username as accessToken, "
                . "usr_username as fullName, usr_password as password, "
                . "usr_cedula as identificacion "
                . "FROM cmp_user u "
                . "WHERE usr_username = :username AND usr_status = '"
                .Cmpuser::STATUS_ENABLED."'";
        $command = \Yii::$app->db->createCommand($sql);
        $command->bindParam(":username", $username, PDO::PARAM_STR);
        $user = $command->queryOne();

        if ($user) {
            return new static($user);
        }

        return null;
    }

    /**
     * {@inheritdoc}
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * {@inheritdoc}
     */
    public function getAuthKey()
    {
        return $this->authKey;
    }

    /**
     * {@inheritdoc}
     */
    public function validateAuthKey($authKey)
    {
        return $this->authKey === $authKey;
    }

    /**
     * Validates password
     *
     * @param string $password password to validate
     * @return bool if password provided is valid for current user
     */
    public function validatePassword($password)
    {
        $codeEncrypt = \Yii::$app->params['codeEncrypt'];
        $encrypter = new RijndaelHex($codeEncrypt);
        $dencryptedPass = $encrypter->lindecrypt($this->password);
        return $password === $dencryptedPass;
    }
    
    /**
     * Valida si el usaurio tiene perfil de super administrador
     * 
     * @return boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co     
     */
    public function isSuperAdmin() {        
        $arrRoles = \Yii::$app->authManager->getRolesByUser(\Yii::$app->user->getId());        
        if(array_key_exists(Cmpuser::SUPER_ADMIN, $arrRoles)){
            return true;
        }
        return false;
    }
    
    /**
     * Valida si el usaurio tiene perfil de administrador
     * 
     * @return boolean
     * @author    Alexander Arcila <alexander.arcila@ingeneo.com.co>
     * @copyright 2017 INGENEO S.A.S.
     * @license   http://www.opensource.org/licenses/bsd-license.php BSD License
     * @version   Release: $Id$
     * @link      http://www.ingeneo.com.co     
     */
    public function isAdmin() {        
        $arrRoles = \Yii::$app->authManager->getRolesByUser(\Yii::$app->user->getId());
        if(array_key_exists(Cmpuser::ADMIN, $arrRoles)){
            return true;
        }
        return false;
    }
}
