<?php

use yii\helpers\Html;
use yii\widgets\ActiveForm;
use kartik\sortinput\SortableInput;
use kartik\export\ExportMenu;
use kartik\grid\GridView;
use yii\helpers\Url;
use app\assets\PuglinsAsset;
use miloschuman\highcharts\Highcharts;
use kartik\select2\Select2;

PuglinsAsset::register($this);

/* @var $this yii\web\View */
/* @var $model app\models\Reportes */
/* @var $form yii\widgets\ActiveForm */

$btnRefress = Html::a("<i class='glyphicon glyphicon-refresh'></i> "
    .Yii::t('app', 'Refrescar'), 
    ($model->isNewRecord ? ['create'] : ['update', 'id'=>$model->id]), 
    ['class' => 'btn btn-default']);

$dropdownGrafica = [];
if(count($listParamsDefecto)>0){
    foreach ($listParamsDefecto as $key => $value) {
        $dropdownGrafica[$key] = $value['content'];
    }
}
#-------------------------------------------------------------------------------
$selectTipo = !empty($model->tipo)? $model->tipo : '';
$selectEjeX = !empty($model->ejex)? explode(',', $model->ejex) : null;
$selectEjeY = !empty($model->ejey)? explode(',', $model->ejey) : null;
$this->registerJsFile('@web/js/grouped-categories.js', 
        ['position' => \yii\web\View::POS_END, 
            'depends'=> \miloschuman\highcharts\HighchartsAsset::className()]);
?>

<div class="reportes-form box <?= (!$model->isNewRecord)? 'collapsed-box' : '' ?>">
    <?php if(!$model->isNewRecord): ?>
        <div class="box-header with-border">
            <h3 class="box-title"><strong><?= Yii::t('app', 'Parametros') ?></strong></h3>
            <div class="box-tools pull-right">
                <button type="button" class="btn btn-box-tool" data-widget="collapse"><i class="fa fa-plus"></i>
                </button>
            </div>        
        </div>
    <?php endif; ?>    
    <div class="box-body">
    <?php $form = ActiveForm::begin(); ?>

    <?= $form->field($model, 'reporte')->textInput(['maxlength' => true]) ?>

    <h5><strong><?= Yii::t('app', 'Parametros Adicionales') ?></strong></h5>     
        <?= SortableInput::widget([
                'name'=>'listAdd',
                'value'=>$valueListAdd,
                'sortableOptions' => ['type'=>'grid'], 
                'items' => $listParamsAdd,
                'hideInput' => true,
                'sortableOptions' => [
                    'itemOptions'=>['style' => 'min-height: 10%; font-size: 85%;'],   
                    'connected'=>true,
                    'type'=>'grid'
                ],
                'options' => ['class'=>'form-control', 'readonly'=>true]
            ]);
        ?>
    
    <h5><strong><?= Yii::t('app', 'Parametros Reporte') ?></strong></h5>
        <?= SortableInput::widget([
            'name'=>'listDefecto',
            'value'=>$valueListDefecto,
            'sortableOptions' => ['type'=>'grid'], 
            'items' => $listParamsDefecto,
            'hideInput' => true,
            'sortableOptions' => [
                'itemOptions'=>['class'=>'btn btn-primary', 'style' => 'min-height: 10%; font-size: 85%;'],                    
                'connected'=>true,
                'type'=>'grid'
            ],
            'options' => ['class'=>'form-control', 'readonly'=>true]
        ]); ?>

    <?= $form->field($model, 'usuario_id')->hiddenInput(['value'=>\Yii::$app->user->getId()])->label(false)->error(false) ?>
    
    <hr>
    <div class="form-group">
        <div class="col-lg-offset-1 col-lg-11">
            <?= Html::submitButton('<i class="fa fa-save"></i> '
                .($model->isNewRecord ? Yii::t('app', 'Create') : Yii::t('app', 'Update')), ['class' => 'btn btn-primary', 'name'=>'create']) ?>
            
            <?= Html::submitButton('<i class="fa fa-save"></i> '
                .Yii::t('app', 'Visualizar'), ['class' => 'btn btn-danger', 'name'=>'preview']) ?>

            <?= Html::a('<i class="fa fa-arrow-left"></i> '
                .Yii::t('app', 'Cancel'), ['index'], ['class' => 'btn btn-default']) ?>            
        </div>
    </div>

    <?php ActiveForm::end(); ?>

    </div>
</div>
<?php    
    $gridColumns = $columns;                                
    $fullExportMenu = ExportMenu::widget([
        'dataProvider' => $dataProvider,
        'columns' => $gridColumns,
        'columnSelectorOptions' => [
            'label' => Yii::t('app', 'Columnas'),
        ],
        'dropdownOptions' => [
            'label' => Yii::t('app', 'Exportar'),
            'class' => 'btn btn-default'
        ],
        //'fontAwesome' => true,
        'showConfirmAlert' => false,
        'target' => '_blank',
        'filename' => $this->title . '_' . date('Ymd'),
        'exportRequestParam' => 'exportar',
        'columnBatchToggleSettings' => [
            'label' => Yii::t('app', 'Todo')
        ],
        'exportConfig' => [
            ExportMenu::FORMAT_TEXT => false,
            ExportMenu::FORMAT_PDF => [
                'label' => Yii::t('app', 'PDF'),
                'iconOptions' => ['class' => 'text-danger'],
                //'linkOptions' => [],
                'showHeader' => true,
                'showPageSummary' => true,
                'showFooter' => true,
                'showCaption' => true,
                'options' => ['title' => Yii::t('app', 'Formato documento portable')],
                'alertMsg' => Yii::t('app', 'PDF generado para descargar.'),
                'filename' => 'PDF - ' . isset($model->reporte)? $model->reporte : '',
                'mime' => 'application/pdf',
                'extension' => 'pdf',
                'writer' => 'PDF',                
            ],
            ExportMenu::FORMAT_HTML => false,
        ]
    ]);
?>
<div class="reportes-gridw box">     
    <?php
    $layout = <<< HTML
    <div class="box-header with-border" >
    {$fullExportMenu} {toggleData} {$btnRefress}
    </div>
        {summary}
    <div class="clearfix"></div>
        {items}
    {pager}
HTML;
    
    if($preview){
        $layout = <<< HTML
    <div class="box-header with-border" >    
    </div>
        {summary}
    <div class="clearfix"></div>
        {items}    
HTML;
    }
    ?>    
    <div class="box-body">
        <small>
            <?= GridView::widget([                
                'dataProvider' => $dataProvider,
                'filterModel' => $reporteDinamico,
                'responsive' => true,
                'responsiveWrap' => false,
                'showPageSummary' => true,
                'resizableColumns'=>true, 
                'floatHeader'=>false,
                'floatHeaderOptions'=>['scrollingTop'=>'0'],
                'layout' => $layout,
                'toolbar'=> [            
                    '{export}',
                    '{toggleData}',
                ],                    
                'export'=>[
                    'fontAwesome'=>true
                ],           
                'exportConfig' => [            
                    GridView::EXCEL => true,
                    GridView::PDF => true
                ],        
                'columns' => $gridColumns,
            ]); ?>
        </small>
    </div>
</div>
<div class="reportes-gridw box" id="box-graficas">    
    <?php if(!$model->isNewRecord): ?>
        <div class="box-header with-border">
            <h3 class="box-title"><strong><?= Yii::t('app', 'grafica') ?></strong></h3>
            <div class="box-tools pull-right">
                <button type="button" class="btn btn-box-tool" data-widget="collapse">
                    <i class="fa fa-minus"></i>
                </button>
            </div>        
        </div>
    <?php endif; ?> 
    <div class="box-body">
        <div class="row">            
            <div class="col-lg-offset-1 col-md-3">
                <?= Html::label('Tipo', 'tipo', 
                    ['class'=>'control-label', 'style'=>'padding-top: 7px;']) ?>
                <?= Html::dropDownList('tipo', $selectTipo, 
                    ['area'=>'Área', 'line'=>'Línea', 'column'=>'Barras'], 
                    ['class'=>'form-control', 'id'=>'tipo']) ?>
                <?= Html::hiddenInput('id', $model->id, ['id'=>'id_reporte']) ?>
            </div>            
            <div class="col-md-3">                
                <?= Html::label('Eje X', 'ejex', 
                    ['class'=>'control-label', 'style'=>'padding-top: 7px;']) ?>
                <?= Select2::widget([
                    'name' => 'ejex',
                    'id' => 'ejex',
                    'data' => $dropdownGrafica,
                    'value' => $selectEjeX,
                    'theme' => Select2::THEME_DEFAULT,
                    'maintainOrder' => true,
                    'options' => [
                        'placeholder' => 'Seleccione ...', 
                        'multiple' => true
                    ],
                    'pluginOptions' => [
                        'allowClear' => true,
                        'tags' => true,
                        'maximumSelectionLength' => '3'
                    ],
                ])?>
            </div>            
            <div class="col-md-3">                                
                <?= Html::label('Eje Y', 'ejey', 
                    ['class'=>'control-label', 'style'=>'padding-top: 7px;']) ?>
                <?= Select2::widget([
                    'name' => 'ejey',
                    'id' => 'ejey',
                    'data' => $dropdownGrafica,
                    'value' => $selectEjeY,
                    'theme' => Select2::THEME_DEFAULT,
                    'maintainOrder' => false,
                    'options' => [
                        'placeholder' => 'Seleccione ...', 
                        'multiple' => true
                    ],
                    'pluginOptions' => [
                        'allowClear' => true
                    ],
                ])?>
            </div>
            <div class="col-md-2" style="padding-top: 20px">  
            <?= Html::button('<i class="fa fa-area-chart"></i> '
                .Yii::t('app', 'Graficar'), [
                    'class' => 'btn btn-danger', 
                    'name'=>'graficar', 'id'=>'btn-graficar']) ?>
            </div>
        </div>
        <hr>
        <div class="row">
            <div id="grafica-widget" style="height:auto;" class="col-md-12">
                 <?= Highcharts::widget([
                        'scripts' => [
                            'highcharts-more',   
                            'modules/exporting', 
                            //'themes/grid' 
                        ],
                        'id' => 'chart-graficas',
                        'options' => [ 
                            'chart' => [
                                'renderTo' => 'chart-graficas',
                                'height' => '800',
                            ],
                            'lang' => [
                                'downloadJPEG' => 'Descargar en JPG',
                                'downloadPDF' => 'Descargar en PDF',
                                'downloadPNG' => 'Descargar en PNG',
                                'downloadSVG' => 'Descargar en SVG',
                                'printChart' => 'Imprimir',
                                'resetZoom' => 'Reestablecer',
                            ],
                           'title' => ['text' => 'Grafica'],
                           'xAxis' => [
                                'categories' => []                              
                           ],
                           'yAxis' => [
                              'title' => ['text' => 'Eje Y']
                           ],
                           'series' => [
                              ['name' => 'Grafica 1 ', 'data' => []],
                              ['name' => 'Grafica 2', 'data' => []]
                           ]
                        ]
                     ]);                
                ?>                              
            </div>                
        </div>
    </div>
</div>
<script type="text/javascript">
    
    $(document).ready(function(){
        graficar();
    });
    
    function getGrafica(data){
        new Highcharts.chart('chart-graficas', data);
    }

    $('body').on('dragend', 'li', function(){
        $("ul[id$='sortable'").trigger('sortupdate');
    });
    
    $('#btn-graficar').click(function(){
        graficar();
    });     
    
    function graficar(){        
        dataEjex = $('#ejex').val();
        dataEjey = $('#ejey').val();
        id = $('#id_reporte').val();
        tipo = $('#tipo').val();
        if(dataEjex != '' && dataEjey != '' && dataEjey != null){             
            $("#box-graficas").append('<div class="overlay"><i class="fa fa-refresh fa-spin"></i></div>');
            $.ajax({url: '<?= Url::to(['getdatagraficas']) ?>',
                type: "post",
                data: {dataEjex: dataEjex, dataEjey: dataEjey, id: id, tipo: tipo},
                dataType: 'json',
                //async: false,
                error : function(){                
                    alert('<?= Yii::t('app', 'Error en los datos')?>');                    
                    $(".overlay").remove();
                },
                success: function(data) { 
                    getGrafica(data);        
                    $('#grafica-widget').show();
                    $(".overlay").remove();                                                  
                }
            });
        }
    }
</script>