<?php

use yii\helpers\Html;
use kartik\grid\GridView;
use yii\bootstrap\ActiveForm;
use app\models\Reasonresponsable;
use yii\helpers\ArrayHelper;
use kartik\export\ExportMenu;
use kartik\grid\EditableColumn;
use miloschuman\highcharts\Highcharts;
use miloschuman\highcharts\HighchartsAsset;

/* @var $this yii\web\View */
/* @var $searchModel app\models\PendingSearch */
/* @var $dataProvider yii\data\ActiveDataProvider */

$this->title = Yii::t('app', 'Pendings');
$this->params['breadcrumbs'][] = $this->title;
$reasons = ArrayHelper::map(Reasonresponsable::find()
        ->select(['id',"CONCAT(reason,' - ', responsable) as reason"])
        ->orderby('reason')->all(), 'id', 'reason');

$btnRefress = Html::a("<i class='glyphicon glyphicon-refresh'></i> "
    .Yii::t('app', 'Refrescar'), 
     ['index'], 
    ['class' => 'btn btn-default']);
?>
<div class="pending-index box box-solid">
    <?php $form = ActiveForm::begin([
        'action' => ['index'],
        'layout' => 'horizontal',
        'fieldConfig' => [
        'horizontalCssClasses' => [
                'label' => 'col-sm-3',
                'offset' => 'col-sm-offset-1',
                'wrapper' => 'col-sm-9',
            ],
        ],
    ]); ?>
    <?php // echo $this->render('_search', ['model' => $searchModel]); ?>
        <div class="box-header with-border">
        <p>
        <div class="col-md-10">
            <?= $form->field($model, 'reasonId')->dropDownList($reasons, ['prompt' => 'Seleccione...']) ?>
        </div>
            <?= Html::submitButton('<i class="fa fa-save"></i> '
                    .Yii::t('app', 'Save'), ['class' => 'btn btn-success', 'name'=>'create']) ?>
        </p>
    </div>  
    <div class="box-body">
        <small>
            <?php 
            //NO QUITAR --- EL BROWSER NO PERMITE FORM DENTRO DE OTRO FORM
            //ES NECESARIO TENER UN FORM FAKE PARA QUE NO ALTERE EL EDTABLE DEL GRID
            ActiveForm::begin();
            ActiveForm::end(); ?>
            <?php 
                                        
                $columns = [
                    [
                        'class' => 'kartik\grid\CheckboxColumn',
                        'headerOptions' => ['class' => 'kartik-sheet-style'],
                        'pageSummary' => '<small>(amounts in $)</small>',
                        'pageSummaryOptions' => ['colspan' => 3, 'data-colspan-dir' => 'rtl']
                    ],
                    'id',
                    'nit',
                    'business_name',
                    'branch_name',
                    'number_order',
                    'reference',
                    'description',
					'operation_center',
                    [
                        'attribute' => 'quantity',  
                        'format'=>['decimal',0]
                    ],
                    [
                        'attribute' => 'cost',  
                        'format'=>['decimal',0]
                    ],
                    'reason',
                    [
                        'class' => 'kartik\grid\EditableColumn',
                        'attribute' => 'detail',  
                        'hAlign' => 'center',
                        'vAlign' => 'middle',
                    ],
                    'responsable',
                    [
                        'attribute' => 'created',
                        //'format' => ['date', 'php: Y M d'],
                        'value' => function($data){
                            return date("Y-m-d", strtotime($data->created));  
                        },
                        'filterType' => GridView::FILTER_DATE_RANGE,
                        'filterWidgetOptions' => [
                            'options' => ['placeholder' => 'Rango Fechas', 'readonly'=>true],
                            'pluginOptions' => [
                                'allowClear' => true,
                                'locale'=>[                            
                                    'format' => 'YYYY-MM-DD',
                                    'separator'=>' - ',
                                ],                        
                            ],
                        ],                
                    ],
                    [
                        'class' => 'yii\grid\ActionColumn',
                        'template' => '{view}{delete}'
                    ],
                ];
                        
                $fullExportMenu = ExportMenu::widget([
                    'dataProvider' => $dataProvider,
                    'columns' => $columns,
                    'columnSelectorOptions' => [
                        'label' => Yii::t('app', 'Columnas'),
                    ],
                    'dropdownOptions' => [
                        'label' => Yii::t('app', 'Exportar'),
                        'class' => 'btn btn-default'
                    ],
                    //'fontAwesome' => true,
                    'showConfirmAlert' => false,
                    'target' => '_blank',
                    'filename' => $this->title . '_' . date('Ymd'),
                    'exportRequestParam' => 'exportar',
                    'columnBatchToggleSettings' => [
                        'label' => Yii::t('app', 'Todo')
                    ],
                    'exportConfig' => [
						ExportMenu::FORMAT_TEXT => false,
						ExportMenu::FORMAT_PDF => [
							'label' => Yii::t('app', 'PDF'),
							'iconOptions' => ['class' => 'text-danger'],
							//'linkOptions' => [],
							'showHeader' => true,
							'showPageSummary' => true,
							'showFooter' => true,
							'showCaption' => false,
							'options' => ['title' => Yii::t('app', 'Formato documento portable')],
							'alertMsg' => Yii::t('app', 'PDF generado para descargar.'),
							'filename' => 'PDF - ' . isset($model->reason)? $model->reason : '',
							'mime' => 'application/pdf',
							'extension' => 'pdf',
							'writer' => 'PDF',                
						],
						ExportMenu::FORMAT_HTML => false,
						ExportMenu::FORMAT_EXCEL => false,
					]
                ]);
            ?>
<?php
    $layout = <<< HTML
    <div class="box-header with-border" >
    {$fullExportMenu} {toggleData} {$btnRefress}
    </div>
        {summary}
    <div class="clearfix"></div>
        {items}
    {pager}
HTML;
    
    ?> 

        <?= GridView::widget([                
                'dataProvider' => $dataProvider,
                'filterModel' => $searchModel,
                'responsive' => true,
                'responsiveWrap' => false,
                'showPageSummary' => false,
                'resizableColumns'=>true, 
                'floatHeader'=>false,
                'floatHeaderOptions'=>['scrollingTop'=>'0'],
                'layout' => $layout,
                'toolbar'=> [            
                    '{export}',
                    '{toggleData}',
                ],                    
                'export'=>[
                    'fontAwesome'=>true
                ],           
                'exportConfig' => [            
                    GridView::EXCEL => true,
                    GridView::PDF => true
                ],        
                'columns' => $columns,
            ]); ?>
        </small>
    </div>
    <?php ActiveForm::end(); ?>
</div>
<div class="box box-solid"> 
    <div class="box-body"> 
    <?= Highcharts::widget([
            'scripts' => [
                'highcharts-more',   
                'modules/exporting', 
                //'themes/grid' 
            ],
            'id' => 'chart-pendientes',
            'options' => [ 
                'chart' => [
                    'renderTo' => 'chart-pendientes',
                    'height' => '500',
                ],
                'lang' => [
                    'downloadJPEG' => 'Descargar en JPG',
                    'downloadPDF' => 'Descargar en PDF',
                    'downloadPNG' => 'Descargar en PNG',
                    'downloadSVG' => 'Descargar en SVG',
                    'printChart' => 'Imprimir',
                    'resetZoom' => 'Reestablecer',
                ],
               'title' => ['text' => 'Porcentaje Motivos (Pendientes)'],
               'xAxis' => [
                    'categories' => $dataGraph['categories']                             
               ],
               'yAxis' => [
                  'title' => ['text' => 'Porcentaje']
               ],
                'tooltip' => [
                    'headerFormat'=> '<span style="font-size:11px">{series.name}</span><br>',
                    'pointFormat'=> '<span style="color:{point.color}">{point.name}</span>: <b>{point.y:.2f}%</b> del Total<br/>'
                ],          
                'plotOptions' => [
                    'column' => [
                        'dataLabels' => [
                            'enabled'=>true,
                            'format'=>'{point.y:,.2f} %',
                        ],
                        'center' => ['50%', '50%'],                    
                        'enableMouseTracking' => true,                    
                    ],                                                           
                ],
               'series' => [
                    [
                      'type' => 'column',
                      'colorByPoint' => true, 
                      'name' => 'Motivos', 
                      'data' => $dataGraph['y']
                    ],
               ]
            ]
         ]);                
    ?>
    </div>
</div>
