<?php

use yii\helpers\Html;
use yii\widgets\ActiveForm;
use app\models\Purchases;
use yii\web\JsExpression;
use kartik\depdrop\DepDrop;
use yii\helpers\Url;
use kartik\date\DatePicker;
use kartik\select2\Select2;

/* @var $this yii\web\View */
/* @var $model app\models\Purchases */
/* @var $form yii\widgets\ActiveForm */
$branch = [];
?>

<div id="purchases-form" class="purchases-form box box-solid">
    <div class="box-body">
        <div id="div-error" style="display: none">
            <button class="close" type="button" data-dismiss="alert" aria-hidden="true">×</button>
            <h4><i class="icon fa fa-ban"></i>Alerta!</h4>
            <div id="msg-alert"></div>
        </div>
        <?php $form = ActiveForm::begin([
            'id' => 'form-purchases',
            'action' => ['purchases/getitems'],
            'method' => 'post',
            'validationUrl' => ['purchases/validateform'],
            'validateOnSubmit' => true,
            'enableAjaxValidation' => true,
            ]); ?>
        <div class="row">
            <div class="col-md-6">
                <?= $form->field($model, 'nit')->widget(Select2::classname(), [    
                //'initValueText' => $provider, 
                //'data' => Purchases::selectProviders(), 
                'options' => [
                    'placeholder' => 'Seleccione Proveedor...',
                ],
                'theme' => Select2::THEME_DEFAULT,
                'pluginOptions' => [
                    'allowClear' => true,
                    'language' => [
                        'errorLoading' => new JsExpression("function () { return 'Buscando...'; }"),
                    ],
                    'minimumInputLength' => 3,
                    'language' => 'es',
                    'ajax' => [
                        'url' => $url = Url::to(['purchases/getlistproviders']),
                        'dataType' => 'json',
                        'data' => new JsExpression('function(params) { return {q:params.term}; }'),                        
                    ],
                    'escapeMarkup' => new JsExpression('function (markup) {return markup; }'),
                    'templateResult' => new JsExpression('function(data) { return data.text; }'),
                    'templateSelection' => new JsExpression('function (data) { '
                        . 'tmpText = data.text;'
                        . 'text = tmpText.split(" - ")[1];'
                        . '$("#purchases-business_name").val(text); '
                        . 'return data.text; }'),
                ],
                ])->label(Yii::t('app', 'Nit - Razon social'));?>
                <?= $form->field($model, 'business_name')->hiddenInput()->label(false)->error(false); ?>
            </div>
            <div class="col-md-6">
                <?= $form->field($model, 'branch_code')->widget(DepDrop::classname(), [                    
                'type'=>DepDrop::TYPE_SELECT2, 
                'select2Options'=>[
                    'theme' => Select2::THEME_DEFAULT,
                    'pluginOptions'=>[
                        'allowClear'=>true, 
                        'escapeMarkup' => new JsExpression('function (markup) {return markup; }'),
                        'templateResult' => new JsExpression('function(data) { return data.text; }'),
                        'templateSelection' => new JsExpression('function (data) { '
                            . 'tmpText = data.text;'
                            . 'text = tmpText.split(" - ")[1];'
                            . '$("#purchases-branch_name").val(text); '
                            . 'return data.text; }'),
                    ],                            
                 ],
                'pluginOptions'=>[
                    'initialize' => true,
                    'initDepends'=>['purchases-nit'],
                    'depends'=>['purchases-nit'],
                    'placeholder' => 'Seleccione...',
                    'loadingText' => 'Buscando Sucursales...',
                    'url' => Url::to(['getlistbranchs', 'select'=>$branch])
                ]]);
                ?>

                <?= $form->field($model, 'branch_name')
                    ->hiddenInput()->label(false)->error(false); ?>
            </div>
        </div>
        <div class="row">
            <div class="col-md-6">
                <?= $form->field($model, 'cellar')->widget(Select2::classname(), [    
                //'initValueText' => $provider, 
                'data' => Purchases::selectCellars(), 
                'options' => [
                    'placeholder' => 'Seleccione la bodega...',
                ],
                'theme' => Select2::THEME_DEFAULT,
                'pluginOptions' => [
                    'allowClear' => true,
                    'language' => [
                        'errorLoading' => new JsExpression("function () { return 'Buscando...'; }"),
                    ],
                    //'minimumInputLength' => 3,
                    'language' => 'es',
                    'escapeMarkup' => new JsExpression('function (markup) {return markup; }'),
                    'templateResult' => new JsExpression('function(data) { return data.text; }'),
                ],
                ]);?>
            </div>
            <div class="col-md-6">
                <?= $form->field($model, 'projection_day')->textInput(['type' => 'number']) ?>
            </div>
        </div>
        <div class="row">
            <div class="col-md-6">
                <?= $form->field($model, 'start_date')->widget(
                        DatePicker::classname(), [
                            'options' => [
                                'placeholder' => 'Fecha inicial ventas ...',
                                'autocomplete'=>'off',
                                'readOnly'=>true
                            ],
                            'pluginOptions' => [
                                'autoclose' => true,
                                'format' => 'yyyy-mm-dd'
                            ]
                    ]);
                ?>
            </div>
            <div class="col-md-6">
                <?= $form->field($model, 'end_date')->widget(
                        DatePicker::classname(), [
                            'options' => [
                                'placeholder' => 'Fecha final ventas ...',
                                'autocomplete'=>'off',
                                'readOnly'=>true
                            ],
                            'pluginOptions' => [
                            'autoclose' => true,
                            'format' => 'yyyy-mm-dd'
                        ]
                    ]);
                ?>
            </div>
        </div>        
        <div id="buttom" class="form-group">
            <?= Html::submitButton('<i class="fa fa-search"></i> '
                    .Yii::t('app', 'Search'), ['class' => 'btn btn-primary']) ?>
            <?= Html::a('<i class="fa fa-reply"></i> '
                    .Yii::t('app', 'Cancel'), ['index'], ['class' => 'btn btn-default']) ?>
        </div>
    </div>
    <?php ActiveForm::end(); ?>
    <div id="form-items"></div>    
</div>
<?php
$this->registerJsFile(
     '@web/js/purchases/forms-purchases.js'
);
?>

<script>
    
    $(document).ready(function () {
        $('form#form-purchases').on('beforeSubmit', function (){
            var form = $(this);
            // return false if form still have some validation errors
            if (form.find('.has-error').length) {
                return false;
            }
            // submit form -----------------------------------------------------
            getItems(form);
            return false;
        });
    });
    
    /**
     * Retorna los items      
     * @returns string
     */
    function getItems(form) { 
        cleanError();
        showLoading();
        $("#form-items").html(' ');
        withPage = $(document).width();
        $.ajax({
            url    : form.attr('action'),
            type   : 'post',
            data   : form.serialize()+"&withPage="+withPage,
            error : function () {
                showError('<?= Yii::t('app', 'Error Items')?>');
                hideLoading();
            },
            success : function (data) {
                $("#form-items").html(data);
                $('.amount').focus(function() {                    
                    if($(this).attr('readonly') != "readonly" ){
                        defaultText = $(this).val();
                        $(this).val('');
                        var stringTemp = $(this).attr('id');
                        stringSplit = stringTemp.split('_');
                        iden = stringSplit[2];
                        $('#' + iden + '_tr').attr('class', 'danger');
                        //$('#' + iden + '-sub').html(0);
                        //$('#' + iden + '-sub-iva').html(0);
                        //$('#' + iden + '_sub_input_id').val(0);
                    }                     
                });    
                hideLoading();
            }
        });    
    }
    
    /**
     * Selecciona las referencias con cantidades y muestra resumen del pedido
     * @returns {undefined}
     */
    function liquidar() {        
        showLoading();               
        //Validamos seleciconar al menos una referencia ------------------------
        if(valItemSeleciconado()){            
            hideInputsEmpty();
            cleanError();
            buttomSend('send', '<?= Url::to(['changebuttom']) ?>'); 
            hideLoading();
        }else{
            hideLoading();
        }
    }
    
    
    /**
     * Funcion para salir  
     * @returns {undefined}     
     */
    function exitOrder() { 
        inputStatus = '<?= Html::hiddenInput('status', Purchases::STATUS_PENDING) ?>';
        $('#div-status').html(inputStatus);
        save();
    }

    /**
     * Funcion que permite habilitar los campos 
     * para guardar la informacion
     * @returns {Boolean}
     */
    function save() {
        showLoading();
        //enabledForm('purchases-form');
        hideInputsEmpty();
        submitForm('purchases-form');
        hideLoading();
    }
         
    /**
     * funcion para autorizar las peticiones
     * @returns {Boolean}
     */
    function allowOrder() {
        showLoading();
        inputStatus = '<?= Html::hiddenInput('status', Purchases::STATUS_ALLOW) ?>';
        $('#div-status').html(inputStatus);               
        //enabledForm('purchases-form');
        hideInputsEmpty();
        submitForm('purchases-form');
        hideLoading();
    }  

    /**
     * funcion para solicitar autorizacion de las peticiones
     * @returns {Boolean}
     */
    function changetStatus() {
        showLoading();
        inputStatus = '<?= Html::hiddenInput('status', Purchases::STATUS_PENDING)?>';
        $('#div-status').html(inputStatus);
        enabledForm('purchases-form');
        hideInputsEmpty();
        submitForm('purchases-form');
    }
    
    /**
     * Funcion que habilita la edicion de la informacion
     * @returns {undefined}     
     * */
    function edit() {
        showTabs();
        $('#requisition-nit').select2("enable");
        //$('#requisition-code_branch').select2("enable",true);
        //$('#summary').hide();
        buttomSend('liquidar', '<?= Url::to(['changebuttom']) ?>');
    }
    
</script>
